# -*- coding: utf-8 -*-
"""
Copyright (C) 2009-2013 All Rights Reserved. Luiz Fernando Maciel França
              www.siginformatica.com.br <luiz[at]siginformatica.com.br>
                                -------

Copyleft (CC) 2013, Creative Commons
Autor - Pierre Geraldo Moura Barbosa <barbosa.pierre[at]gmail.com>
================================================================================
Atribuicao-Nao Comercial-Compartilhada Igual 3.0 Nao Adaptada (CC BY-NC-SA 3.0)
http://creativecommons.org/licenses/by-nc-sa/3.0/deed.pt_BR
================================================================================
"""

from openerp.osv import osv
from datetime import date
from dateutil.parser import parse


class recebimento_itens(osv.Model):
    """
    Heranca em Recebimento. Verificacao do shelf_life

    @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
    """

    _name = 'recebimento.itens'  # recite
    _inherit = 'recebimento.itens'  # recebimento/control/recebimento_itens

    def recite_onchange_data_validade(self, cr, uid, ids, data_validade):
        """
        Verifica data de validade.

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param data_validade: {date} data da validade do produto que sera verificada,
        @return: {dict} mensagem de erro se a data de validade estiver invalida.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        if data_validade:
            result = self.recite_shelf_life(cr, uid, ids, data_validade)

            msg = None
            if result == 1:
                msg = 'Data de validade com Shelf Life incorreto.'
            elif result == 2:
                msg = 'Produto sem validade em dias.'

            if msg:
                return {'warning': {'title': 'Aviso.',  'message': msg}}

        return {}

    def recite_verifica_divergencias(self, cr, uid, ids, context=None):
        """
        Verifica se existe algum item com divergencia na contagem

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {list} lista dos IDs cuja data de validade apresenta algum tipo de divergencia.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # Itens ja considerados divergentes
        result = super(recebimento_itens, self).recite_verifica_divergencias(cr, uid, ids, context)

        # Itens nao divergentes
        nao_divergente_ids = filter(lambda i: i not in result, ids)

        # Verifica Shelf Life
        for item in self.browse(cr, uid, nao_divergente_ids):
            if self.recite_shelf_life(cr, uid, item.id, item.data_validade, context):
                result.append(item.id)

        return result

    def recite_shelf_life(self, cr, uid, ids, validade, context=None):
        """
        Verifica Shelf-Life do produto

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param validade: {date} data da validade do produto que sera verificada,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {list} lista dos IDs cuja data de validade apresenta algum tipo de divergencia.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        try:
            if validade:
                delta = parse(validade, yearfirst=True).date() - date.today()

                for item in self.browse(cr, uid, ids):
                    if item.produto_id.validade:
                        if item.grade_id.shelf_life:
                            dias_shelf_life = item.produto_id.validade * \
                                item.grade_id.shelf_life / 100
                            if (delta.days < dias_shelf_life) or delta.days < 0:
                                return 1  # Shelf Life incorreto
                    else:
                        return 2  # Produto sem validade em dias
        except:
            return 3  # Data de validade nao informada ou a data e invalida

        return 0  # Shelf_life OK.

    def recite_imprimir_dado_logistico(self, cr, uid, ids, context=None):
        """
        Imprime relatorio de produtos faltando algum dado logistico

        @param cr: {cursor} a linha atual do cursor do banco de dados,
        @param uid: {int} ID do usuario utilizado para a verificacao de perfil,
        @param ids: {list} lista dos IDs,
        @param context: {dict} um dicionario padrao para valores contextuais,
        @return: {dict} relatorio de produtos com dado logistico faltando.

        @author: Pierre Barbosa <pierre.barbosa[at]siginformatica.com.br>
        """

        context = context and context.copy() or {}

        if ids and isinstance(ids, (int, long)):
            ids = [ids]

        # Produtos com dado logistico faltando
        datas = {
            'ids': map(lambda g: g.id, filter(lambda g: g.recite_ver_falta_dado_logistico,
                                              self.browse(cr, uid, ids))),
            'model': 'estoque.produto.grade', 'form': {}}

        result = {'type': 'ir.actions.report.xml', 'report_name': 'lista.produto.dados_logisticos',
                'datas': datas, 'context': {'subtitulo': 'LISTA DE PRODUTOS COM DADO LOGISTICO '
                                                         'FALTANDO'}}

        return result

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
